local ext = require "shellExtension" 
assert(shellExtension)

local function checkEnvVar(varname, optional)
    local varvalue = os.getenv(varname)
    if varvalue then
        print("[CONF] Variável do sistema "..varname.." tem valor: "..varvalue) 
    elseif not optional then
        print("\t[ERRO] Variável mandatória do sistema "..varname.." não foi encontrada")
        os.exit(-1)
    end
    io.stdout:flush()
    return varvalue
end

local function printSeparator(title)
    print(" ")  
    print("****************************** "..title.." *************************************")   
    print(" ")
    io.stdout:flush()
end

function string:split(pat)
    local st, g = 1, self:gmatch("()("..pat..")")

    local function getter(self, segs, seps, sep, cap1, ...)
        st = sep and seps + #sep
        return self:sub(segs, (seps or 0) - 1), cap1 or sep, ...
    end
    
    local function splitter(self)
        if st then return getter(self, st, g()) end
    end
    
    return splitter, self
end

local function isFlow()
    local flow = os.getenv("CSBASE_FLOW_GUILTY_NODE_ID_LOG")
    if flow then
        return true
    else 
        return false
    end
end

local function checkVars()

    printSeparator("Verificando variáveis de ambiente")

    local envvars = { "CSBASE_BIN_DIR", "CSBASE_PROJ_DIR", "CSBASE_EXECUTION_DIR", "CSBASE_SANDBOX_DIR", "CSBASE_LOG_FILE" }
    for i = 1, #envvars do
        checkEnvVar(envvars[i])
    end
    
    print(" ") 
    
    local optionalvars = { "CSBASE_INPUT_FILES", "CSBASE_OUTPUT_FILES" }
    for i = 1, #optionalvars do
        checkEnvVar(optionalvars[i], true)
    end

    if isFlow() then
    	local flowvars = { "CSBASE_FLOW_GUILTY_NODE_ID_LOG", "CSBASE_FLOW_NODE_ID", "CSBASE_FLOW_SANDBOX_DIR" }
        for i = 1, #flowvars do
        	checkEnvVar(flowvars[i])
		end
	end
end

local function checkExecutionDir()
    local current_dir = ext.dirGetAbsolutePath(".")
    if not current_dir then
        print("\t[ERRO] Não foi possível determinar diretório corrente: ")
        os.exit(-1)
    else 
        print("[INFO] Executando do diretório: "..current_dir)
    end

    local execution_path = os.getenv("CSBASE_EXECUTION_DIR")
    if not ext.isDir(execution_path) then
        print("\t[ERRO] Diretório de execução inválido!")
        os.exit(-1)
    end
    if current_dir ~= execution_path then
        print("\t[ERRO] Diretório de execução é diferente do diretório corrente!")
        os.exit(-1)
    end
    io.stdout:flush()
    
end

local function writeProgress(progress)
	local sandbox_path = os.getenv("CSBASE_SANDBOX_DIR")
    local progressFile = sandbox_path.."/collect_execution_data.log"
    io.output(progressFile)
    io.write("progresso="..progress.."%\n")
end

local function checkSandboxDir(sandbox_path, name)
    if not ext.isDir(sandbox_path) then
        print("\t[ERRO] Diretório de sandbox "..name.." inválido!")
        os.exit(-1)
    end
    
    local sandbox_test_file = sandbox_path.."/csbase_sandbox_test.txt"
    io.output(sandbox_test_file)
    io.write(os.date("%c"),'\n',"Sandbox "..name..": ", sandbox_path,'\n')
    print("[INFO] Arquivo de teste de sandbox "..name.." criado em: "..sandbox_test_file)
    io.stdout:flush()
end

local function checkProjectDir()
    local project_path = os.getenv("CSBASE_PROJ_DIR")
    if not ext.isDir(project_path) then
        print("\t[ERRO] Diretório de projeto inválido!")
        os.exit(-1)
    end
end

local function checkBinDir()
    local bin_path = os.getenv("CSBASE_BIN_DIR")
    if not ext.isDir(bin_path) then
        print("\t[ERRO] Diretório de binários inválido!")
        os.exit(-1)
    end
end

local function checkLogFile()
    local log_file = os.getenv("CSBASE_LOG_FILE")
    if not ext.fileExists(log_file) then
        print("\t[ERROR] Arquivo de log não encontrado")
        os.exit(-1)
    else
        print("[INFO] Arquivo de log em "..log_file)
    end
end


local function checkFiles()
    local i = 1
    local file_contents = {}
    local hasInputFile = false
    while(ext.getParameterValue(arg, 'ENTRADA'..i)) do
        local param_value = ext.getParameterValue(arg, 'ENTRADA'..i)
        local content = ""
        for file_name in param_value:split(",") do      
            print("[INFO] ENTRADA"..i..": "..file_name)
            io.input(file_name)
            content = content.."\n**"..file_name.."**\n"..io.read("*all")
        end
        file_contents[i] = content
        hasInputFile = true
        i = i + 1
    end
    
    local hasOutputFile = false
    i = 1
    while(ext.getParameterValue(arg, 'SAIDA'..i)) do
        local param_value = ext.getParameterValue(arg, 'SAIDA'..i)
        for file_name in param_value:split(",") do      
            print("[INFO] SAIDA"..i..": "..file_name)
            io.output(file_name)
            if file_contents[i] == nil then
                io.write(os.date("%c"),'\n',"SAIDA PADRAO ",i,'\n')
            else 
                io.write(os.date("%c"),'\n',file_contents[i])   
            end
        end
        hasOutputFile = true
        i = i + 1
    end
    file_contents = nil
    
    if hasInputFile then
        local input_files = os.getenv("CSBASE_INPUT_FILES")
        if not input_files then
            print("\t[ERROR] Variável 'CSBASE_INPUT_FILES' deveria estar definida!")
            os.exit(-1) 
        end
    else
        print("[INFO] Não há arquivos de entrada")
    end
   
    if hasOutputFile then
        local output_files = os.getenv("CSBASE_OUTPUT_FILES")
        if not output_files then
            print("\t[ERROR] Variável 'CSBASE_OUTPUT_FILES' deveria estar definida!")
            os.exit(-1) 
        end
    else
        print("[INFO] Não há arquivos de saída")
    end
end

local function checkDirs()
    local i = 1
    local file_contents = {}
    local hasInputFile = false
    while(ext.getParameterValue(arg, 'DIR_ENTRADA'..i)) do
        local param_value = ext.getParameterValue(arg, 'DIR_ENTRADA'..i)
        local content = ""
        for dir_name in param_value:split(",") do      
            print("[INFO] DIR_ENTRADA"..i..": "..dir_name)
			while true do
				local file_name = ext.listOneFile(dir_name.."/*")
				if file_name and file_name ~= "" then
					print("[INFO] Listando arquivo do DIR_ENTRADA"..i..": "..file_name)
		        	io.input(file_name)
		        	content = content.."\n**"..file_name.."**\n"..io.read("*all")
					break
				end
				ext.sleep(1)
			end
        end
        file_contents[i] = content
        hasInputDir = true
        i = i + 1
    end
    
    local hasOutputDir = false
    i = 1
    while(ext.getParameterValue(arg, 'DIR_SAIDA'..i)) do
        local param_value = ext.getParameterValue(arg, 'DIR_SAIDA'..i)
        for dir_name in param_value:split(",") do      
            print("[INFO] DIR_SAIDA"..i..": "..dir_name)
			local file_name = dir_name.."/saida.txt"
            io.output(file_name)
            if file_contents[i] == nil then
                io.write(os.date("%c"),'\n',"SAIDA PADRAO ",i,'\n')
            else 
                io.write(os.date("%c"),'\n',file_contents[i])   
            end
			print("[INFO] Escrevendo saída em : "..file_name)
        end
        hasOutputDir = true
        i = i + 1
    end
    file_contents = nil
    
    if hasInputDir then
        local input_files = os.getenv("CSBASE_INPUT_FILES")
        if not input_files then
            print("\t[ERROR] Variável 'CSBASE_INPUT_FILES' deveria estar definida!")
            os.exit(-1) 
        end
    else
        print("[INFO] Não há diretórios de entrada")
    end
   
    if hasOutputDir then
        local output_files = os.getenv("CSBASE_OUTPUT_FILES")
        if not output_files then
            print("\t[ERROR] Variável 'CSBASE_OUTPUT_FILES' deveria estar definida!")
            os.exit(-1) 
        end
    else
        print("[INFO] Não há diretórios de saída")
    end 
    
end

local function checkDataCollection()
    local bin_path = os.getenv("CSBASE_BIN_DIR")
    local sandbox_path = os.getenv("CSBASE_SANDBOX_DIR")
    
    local start = bin_path.."/start"
    if not ext.fileExists(start) then
        print("\t[WARN] Script 'start' não encontrado")
    else
        local startLog = sandbox_path.."/start.log"
        if not ext.fileExists(startLog) then
            print("\t[ERROR] Script 'start' não rodou com sucesso")
        else
            print("[INFO] Script 'start' rodou com sucesso!")
        end
    end

    local status = bin_path.."/status"
    if not ext.fileExists(status) then
        print("\t[WARN] Script 'status' não encontrado")
    else
        local statusLog = sandbox_path.."/status.log"
        if not ext.fileExists(statusLog) then
            print("\t[ERROR] Script 'status' não rodou com sucesso (pode não ter tido tempo suficiente)")
        else
            print("[INFO] Script 'status' rodou com sucesso!")
        end
    end
    
    local stop = bin_path.."/stop"
    if not ext.fileExists(stop) then
        print("\t[WARN] Script 'stop' não encontrado")
    else
        -- O script stop só é executado após o final do comando, portanto não há 
        -- como verificar aqui se ele foi bem sucedido.
        print("[INFO] Script 'stop' foi encontrado (só será executado após o final do comando)")
    end
    
    local collect = bin_path.."/collect_execution_data"
    if not ext.fileExists(collect) then
        print("\t[WARN] Script 'collect_execution_data' não encontrado")
    else
        local collectLog = sandbox_path.."/collect_execution_data.txt"
        if not ext.fileExists(collectLog) then
            print("\t[ERROR] Script 'collect_execution_data' não rodou com sucesso (pode não ter tido tempo suficiente)")
        else
            print("[INFO] Script 'collect_execution_data' rodou com sucesso!")
        end
    end
    
end

local function main()

    printSeparator("Iniciando")

    print("[INFO] Começando execução em: "..os.date("%c"))
    
	local nodeId
    if isFlow() then
        print("[INFO] Executando fluxo") 
	  	nodeId = os.getenv("CSBASE_FLOW_NODE_ID")
		if nodeId then
			print("[INFO] Identificador do nó do fluxo: "..nodeId)
		end
    else
        print("[INFO] Executando comando simples") 
    end
    io.stdout:flush()    
    
    checkVars()
    
    printSeparator("Verificando ambiente")
    

    if isFlow() then
    	local flow_sandbox = os.getenv("CSBASE_FLOW_SANDBOX_DIR")
		checkSandboxDir(flow_sandbox, "do fluxo")
		local sandbox = os.getenv("CSBASE_SANDBOX_DIR")
		checkSandboxDir(sandbox, "do nó")
	else
		local sandbox = os.getenv("CSBASE_SANDBOX_DIR")
		checkSandboxDir(sandbox, "simples")
	end
    
	writeProgress(0)
	
    checkExecutionDir()
    checkProjectDir()
    checkBinDir()    	
    checkLogFile()

    printSeparator("Verificando arquivos")
    
    checkFiles()
    checkDirs()
    
    local time = ext.getParameterValue(arg, 'SLEEP')
    if(not time) then
        time = "1"
    end

    local exitCode = ext.getParameterValue(arg, 'EXIT_CODE')
    if(not exitCode) then
        exitCode = "0"
    end

    local nsleeps = ext.getParameterValue(arg, 'NSLEEPS')
    if(not nsleeps) then
        nsleeps = "1"
    end
    
    local cmdId, errorMsg = ext.getParameterValue(arg, 'cmdId')
    assert(errorMsg == nil, errorMsg)

    local bin_path = os.getenv("CSBASE_BIN_DIR")
    local path_to_script = ext.dirGetAbsolutePath(bin_path)
    if not path_to_script then
        print("\t[ERRO] Não foi possível determinar diretório do script: "..bin_path)
        os.exit(-1)
    end 
    local exec = "'"..path_to_script.."/teste'"
    
    printSeparator("Executando")

    local exitStatus = 0
    local result = ext.execCmdLine(ext.SHELL_BASH, exec, nil, true, nsleeps, time, exitCode, cmdId, nodeId)
    
    if not isFlow() then
        printSeparator("Verificando coleta de dados")
        -- Só funciona para comandos simples
        checkDataCollection()
    end
    
    printSeparator("Finalizando")

    if not result then
        if exitCode == "0" then    
           print("\t[ERRO] Falha não esperada na execução")
           exitStatus = -1
        else         
           print("\t[WARN] Falha esperada na execução | Código de saída: " .. exitCode)
           exitStatus = exitCode
        end
    else
        print("[INFO] Execução terminada com sucesso!")
    end
    
    print("[INFO] Terminando execução em: "..os.date("%c"))
    io.stdout:flush()   

	writeProgress(100) 
    
    os.exit(exitStatus)
end

main()
