/*
 * CSGridServer.java
 */
package csgrid.server;

import csbase.logic.ManifestVersion;
import csbase.logic.ServerNotification;
import csbase.logic.ServerURI;
import csbase.server.Server;
import csbase.server.ServerException;
import csbase.server.ServiceManager;
import csbase.server.services.administrationservice.AdministrationService;
import csbase.server.services.algorithmservice.AlgorithmService;
import csbase.server.services.applicationservice.ApplicationService;
import csbase.server.services.commandpersistenceservice.CommandPersistenceService;
import csbase.server.services.csfsservice.CSFSService;
import csbase.server.services.diagnosticservice.DiagnosticService;
import csbase.server.services.diskusageservice.DiskUsageService;
import csbase.server.services.eventlogservice.EventLogService;
import csbase.server.services.ftcservice.FTCService;
import csbase.server.services.httpservice.HttpService;
import csbase.server.services.killserverservice.KillServerService;
import csbase.server.services.logadministrationservice.LogAdministrationService;
import csbase.server.services.loginservice.LoginService;
import csbase.server.services.mailservice.MailService;
import csbase.server.services.messageservice.MessageService;
import csbase.server.services.notificationservice.NotificationService;
import csbase.server.services.openbusservice.OpenBusService;
import csbase.server.services.opendreamsservice.OpenDreamsService;
import csbase.server.services.pluginservice.PluginService;
import csbase.server.services.projectservice.ProjectService;
import csbase.server.services.projectsynchronizationservice.ProjectSynchronizationService;
import csbase.server.services.repositoryservice.RepositoryService;
import csbase.server.services.restservice.RestService;
import csbase.server.services.schedulerservice.SchedulerService;
import csbase.server.services.serverservice.ServerService;
import csbase.server.services.sgaservice.SGAService;
import csbase.server.services.sharedobjectservice.SharedObjectService;

/** A classe <code>CSGridServer</code> representa o servidor. */
public class CSGridServer extends Server {
  /** Versão do servidor. */
  private static final ManifestVersion version = new ManifestVersion(CSGridServer.class);

  /**
   * Constrói o servidor do CSGrid.
   *
   * @param args argymentos.
   * @throws ServerException em caso de falha.
   */
  public CSGridServer(final String[] args) throws ServerException {
    super(args);
  }

  /** {@inheritDoc} */
  @Override
  public void createServices() throws ServerException {
    try {
      RepositoryService.createService();
      PluginService.createService();
      MessageService.createService();
      FTCService.createService();
      AdministrationService.createService();
      AlgorithmService.createService();
      ApplicationService.createService();
      EventLogService.createService();
      HttpService.createService();
      KillServerService.createService();
      MailService.createService();
      NotificationService.createService();
      CSFSService.createService();
      SGAService.createService();
      SchedulerService.createService();
      ServerService.createService();
      SharedObjectService.createService();
      CommandPersistenceService.createService();
      // O Serviço OpenBus precisa ser criado antes do ProjectService e do
      // OpenDreams
      OpenBusService.createService();
      ProjectService.createService();
      ProjectSynchronizationService.createService();
      OpenDreamsService.createService();
      // O serviço de monitoração de disco deve ser criado após o serviço de
      // projetos.
      DiskUsageService.createService();
      LogAdministrationService.createService();
      DiagnosticService.createService();

      RestService.createService();

      // NewService.createService();
      // LoginService deve ser o último, para garantir que quem faz login
      // com sucesso obtém o conjunto completo de serviços.
      LoginService.createService();
      System.out.println("Serviços iniciados...");
    } catch (Exception e) {
      throw new ServerException("Erro na criação do serviço", e);
    }
  }

  /** {@inheritDoc} */
  @Override
  public void startCentralServerMonitor() {
    ServerURI sURI = ServerURI.parse(getCentralServerName());
    if (sURI == null) {
      throw new IllegalArgumentException("URI do servidor central inválida");
    }
    new CentralServerMonitor(sURI, getIntProperty("monitor.server.window"));
  }

  /** {@inheritDoc} */
  @Override
  public void postInitialization() {
    notifyStartToAllUsers();
  }

  /** Notificação de start. */
  private void notifyStartToAllUsers() {
    try {
      ServiceManager srvManager = ServiceManager.getInstance();
      if (srvManager == null) {
        throw new csbase.server.ServerException("Gerente de Serviços fora do ar");
      }
      NotificationService notSrv = NotificationService.getInstance();
      if (notSrv == null) {
        throw new csbase.server.ServerException("Serviço de notificação fora do ar");
      }
      notSrv.notifyTo(null, new ServerNotification("CSGridServer", true));
    } catch (Exception e) {
      String errMsg = "Falha de notificação na inicialização!";
      Server.logSevereMessage(errMsg, e);
    }
  }

  /** {@inheritDoc} */
  @Override
  public String getVersion() {
    return version.getVersionName();
  }

  /**
   * Inicia o servidor do CSGrid.
   *
   * @param args Não está sendo utilizado.
   */
  public static void main(String[] args) {
    try {
      CSGridServer server = new CSGridServer(args);
      boolean success = server.start();
      if (!success) {
        System.out.println(
            "Não foi possível iniciar o servidor."
                + "\nPor favor consulte o log para maiores detalhes.");
        System.exit(1);
      }
      ServiceManager srvManager = ServiceManager.getInstance();
      if (srvManager == null) {
        System.out.println("Gerente de Serviços fora do ar");
      }
    } catch (Exception e) {
      e.printStackTrace();
      System.exit(1);
    }
  }
}
